@@ -1,3 +1,7 @@
+{-# LANGUAGE CPP #-}
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+{-# LANGUAGE Safe #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Functor.Apply
@@ -27,4 +31,4 @@ module Data.Functor.Apply (
   , MaybeApply(..)
   ) where
 
-import Data.Functor.Bind 
+import Data.Functor.Bind
