@@ -1,3 +1,15 @@
+{-# LANGUAGE CPP #-}
+#ifndef MIN_VERSION_comonad
+#define MIN_VERSION_comonad(x,y,z) 1
+#endif
+
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 707 && (MIN_VERSION_comonad(3,0,3))
+{-# LANGUAGE Safe #-}
+#else
+{-# LANGUAGE Trustworthy #-}
+#endif
+#endif
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 -----------------------------------------------------------------------------
 -- |
@@ -40,6 +52,9 @@ import Control.Applicative
 import Control.Arrow
 import Control.Category
 import Control.Comonad
+import Control.Comonad.Trans.Env
+import Control.Comonad.Trans.Store
+import Control.Comonad.Trans.Traced
 import Control.Monad (ap)
 import Control.Monad.Instances
 import Control.Monad.Trans.Cont
@@ -227,6 +242,15 @@ instance (Bind m, Semigroup w) => Apply (Lazy.RWST r w s m) where
 instance Apply (ContT r m) where
   ContT f <.> ContT v = ContT $ \k -> f $ \g -> v (k . g)
 
+instance (Semigroup e, Apply w) => Apply (EnvT e w) where
+  EnvT ef wf <.> EnvT ea wa = EnvT (ef <> ea) (wf <.> wa)
+
+instance (Apply w, Semigroup s) => Apply (StoreT s w) where
+  StoreT ff m <.> StoreT fa n = StoreT ((<*>) <$> ff <.> fa) (m <> n)
+
+instance Apply w => Apply (TracedT m w) where
+  TracedT wf <.> TracedT wa = TracedT (ap <$> wf <.> wa)
+
 -- | Wrap an 'Applicative' to be used as a member of 'Apply'
 newtype WrappedApplicative f a = WrapApplicative { unwrapApplicative :: f a }
 
@@ -449,7 +473,7 @@ instance ArrowApply a => Bind (WrappedArrow a b) where
 instance Ord k => Bind (Map k) where
   m >>- f = Map.mapMaybeWithKey (\k -> Map.lookup k . f) m
 
--- | An 'IntMap' is a 'Applicative', but it is an instance of 'Bind'
+-- | An 'IntMap' is not a 'Monad', but it is an instance of 'Bind'
 instance Bind IntMap where
   m >>- f = IntMap.mapMaybeWithKey (\k -> IntMap.lookup k . f) m
 
